@@ -34,6 +34,7 @@ extern "C" {
 #endif
 
 #define GSP_KEY_FILE_DESKTOP_KEY_AUTOSTART_ENABLED "X-MATE-Autostart-enabled"
+#define GSP_KEY_FILE_DESKTOP_KEY_DELAY             "X-MATE-Autostart-Delay"
 
 void      gsp_key_file_populate        (GKeyFile *keyfile);
 
@@ -46,6 +47,8 @@ gboolean gsp_key_file_get_boolean      (GKeyFile       *keyfile,
                                         gboolean        default_value);
 #define gsp_key_file_get_string(key_file, key) \
          g_key_file_get_string (key_file, G_KEY_FILE_DESKTOP_GROUP, key, NULL)
+#define gsp_key_file_get_delay(key_file) \
+         g_key_file_get_integer (key_file, G_KEY_FILE_DESKTOP_GROUP, GSP_KEY_FILE_DESKTOP_KEY_DELAY, NULL)
 #define gsp_key_file_get_locale_string(key_file, key) \
          g_key_file_get_locale_string(key_file, G_KEY_FILE_DESKTOP_GROUP, key, NULL, NULL)
 
@@ -53,6 +56,8 @@ gboolean gsp_key_file_get_boolean      (GKeyFile       *keyfile,
          g_key_file_set_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, key, value)
 #define gsp_key_file_set_string(key_file, key, value) \
          g_key_file_set_string (key_file, G_KEY_FILE_DESKTOP_GROUP, key, value)
+#define gsp_key_file_set_delay(key_file, value) \
+        g_key_file_set_integer(key_file, G_KEY_FILE_DESKTOP_GROUP, GSP_KEY_FILE_DESKTOP_KEY_DELAY, value)
 void    gsp_key_file_set_locale_string (GKeyFile    *keyfile,
                                         const gchar *key,
                                         const gchar *value);
