@@ -26,9 +26,10 @@
 
 #include <gtk/gtk.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+G_BEGIN_DECLS
+
+#define GSM_TYPE_LOGOUT_DIALOG         (gsm_logout_dialog_get_type ())
+G_DECLARE_FINAL_TYPE (GsmLogoutDialog, gsm_logout_dialog, GSM, LOGOUT_DIALOG, GtkMessageDialog)
 
 enum
 {
@@ -40,38 +41,11 @@ enum
         GSM_LOGOUT_RESPONSE_SLEEP
 };
 
-#define GSM_TYPE_LOGOUT_DIALOG         (gsm_logout_dialog_get_type ())
-#define GSM_LOGOUT_DIALOG(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_LOGOUT_DIALOG, GsmLogoutDialog))
-#define GSM_LOGOUT_DIALOG_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GSM_TYPE_LOGOUT_DIALOG, GsmLogoutDialogClass))
-#define GSM_IS_LOGOUT_DIALOG(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_LOGOUT_DIALOG))
-#define GSM_IS_LOGOUT_DIALOG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GSM_TYPE_LOGOUT_DIALOG))
-#define GSM_LOGOUT_DIALOG_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GSM_TYPE_LOGOUT_DIALOG, GsmLogoutDialogClass))
-
-typedef struct _GsmLogoutDialog         GsmLogoutDialog;
-typedef struct _GsmLogoutDialogClass    GsmLogoutDialogClass;
-typedef struct _GsmLogoutDialogPrivate  GsmLogoutDialogPrivate;
-
-struct _GsmLogoutDialog
-{
-        GtkMessageDialog        parent;
-
-        GsmLogoutDialogPrivate *priv;
-};
-
-struct _GsmLogoutDialogClass
-{
-        GtkMessageDialogClass  parent_class;
-};
-
-GType        gsm_logout_dialog_get_type   (void) G_GNUC_CONST;
-
 GtkWidget   *gsm_get_logout_dialog        (GdkScreen           *screen,
                                            guint32              activate_time);
 GtkWidget   *gsm_get_shutdown_dialog      (GdkScreen           *screen,
                                            guint32              activate_time);
 
-#ifdef __cplusplus
-}
-#endif
+G_END_DECLS
 
 #endif /* __GSM_LOGOUT_DIALOG_H__ */
